-- @docclass
g_tooltip = {}

-- private variables
local toolTipLabel
local currentHoveredWidget

g_advancedTooltip = {}
local currentAdvancedHoveredWidget

local advancedTooltipWindow


local function advancedMoveToolTip(first)
  if not first and (not advancedTooltipWindow:isVisible() or advancedTooltipWindow:getOpacity() < 0.1) then return end
  local pos = g_window.getMousePosition()
  pos.x = g_window.getSize().width/2 < pos.x and pos.x - advancedTooltipWindow:getWidth() - 3 or pos.x + 3
  pos.y = g_window.getSize().height/2 < pos.y and pos.y - advancedTooltipWindow:getHeight() - 3 or pos.y + 3
  advancedTooltipWindow:setPosition(pos)
end

-- private functions
local function moveToolTip(first)
    if not first and
        (not toolTipLabel:isVisible() or toolTipLabel:getOpacity() < 0.1) then
        return
    end

    local pos = g_window.getMousePosition()
    local windowSize = g_window.getSize()
    local labelSize = toolTipLabel:getSize()

    pos.x = pos.x + 1
    pos.y = pos.y + 1

    if windowSize.width - (pos.x + labelSize.width) < 10 then
        pos.x = pos.x - labelSize.width - 3
    else
        pos.x = pos.x + 10
    end

    if windowSize.height - (pos.y + labelSize.height) < 10 then
        pos.y = pos.y - labelSize.height - 3
    else
        pos.y = pos.y + 10
    end

    toolTipLabel:setPosition(pos)
end

local function onWidgetHoverChange(widget, hovered)
    if hovered then
        if widget.tooltip and not g_mouse.isPressed() then
            g_tooltip.display(widget.tooltip)
            currentHoveredWidget = widget
        end
        if currentAdvancedHoveredWidget ~= widget and widget.advancedTooltip and not g_mouse.isPressed(MouseLeftButton) then
          currentAdvancedHoveredWidget = widget
          g_advancedTooltip.display()
        end
    else
        if widget == currentHoveredWidget then
            g_tooltip.hide()
            currentHoveredWidget = nil
        end
        if widget == currentAdvancedHoveredWidget then
          g_advancedTooltip.hide()
          currentAdvancedHoveredWidget = nil
        end
    end
end

local function onWidgetStyleApply(widget, styleName, styleNode)
    if styleNode.tooltip then widget.tooltip = styleNode.tooltip end
end

function g_advancedTooltip.setLockCode(v)
  lockCode = v
end

function g_advancedTooltip.init()
  --[[connect(UIWidget, {  onStyleApply = onWidgetStyleApply,
                       onHoverChange = onWidgetHoverChange})]]

  addEvent(function()
    advancedTooltipWindow = g_ui.createWidget('UIWindow', rootWidget)
    advancedTooltipWindow:setId('advancedTooltip')
    advancedTooltipWindow:setImageSource('/images/ui/advancedTooltip')
    advancedTooltipWindow:setImageBorder(5)
    advancedTooltipWindow:setHeight(400)
    advancedTooltipWindow:setWidth(254)
  advancedTooltipWindow:setPhantom(true)
  advancedTooltipWindow:setFocusable(false)

  advancedTooltipName = g_ui.createWidget('HoverNameLabel', advancedTooltipWindow)
  advancedTooltipName:addAnchor(AnchorVerticalCenter, 'parent', AnchorVerticalCenter)
  advancedTooltipName:addAnchor(AnchorHorizontalCenter, 'parent', AnchorHorizontalCenter)
    advancedTooltipName:setId('hoverName')
    advancedTooltipHead = g_ui.createWidget('UIWidget', advancedTooltipWindow)
  advancedTooltipHead:addAnchor(AnchorTop, 'parent', AnchorTop)
  advancedTooltipHead:addAnchor(AnchorHorizontalCenter, 'parent', AnchorHorizontalCenter)
    advancedTooltipHead:setId('head')
    advancedTooltipHead:setImageSource('/images/ui/advancedTooltipHead2')
    advancedTooltipHead:setSize({width = 246, height = 37})
  advancedTooltipHead:setPhantom(true)
    advancedTooltipHead:setMarginTop(5)
 
    advancedTooltipWindow:hide()
    advancedTooltipWindow.onMouseMove = function() advancedMoveToolTip() end
  end)
end

function g_advancedTooltip.terminate()
  currentAdvancedHoveredWidget = nil
  advancedTooltipWindow:destroy()
  advancedTooltipWindow = nil

  g_advancedTooltip = nil
end

function tooltipsTerminate()
  g_tooltip.terminate()
  g_advancedTooltip.terminate()
end

function g_advancedTooltip.display()
  if not advancedTooltipWindow then return end
  advancedTooltipWindow:show()
  advancedTooltipWindow:raise()
  advancedTooltipWindow:setPhantom(true)
  advancedTooltipWindow:enable()
    if tostring(currentAdvancedHoveredWidget.advancedTooltip) then
    local width = 14
    local height = 10
    advancedTooltipWindow:setWidth(500)
    advancedTooltipName:setVisible(true)
    advancedTooltipName:setText(currentAdvancedHoveredWidget.advancedTooltip)
    advancedTooltipName:resizeToText()
    width = width + advancedTooltipName:getWidth()
    height = height + advancedTooltipName:getHeight()
    advancedTooltipWindow:setWidth(width)
    advancedTooltipWindow:setHeight(height)

    advancedTooltipHead:setVisible(false)

  end
  g_effects.fadeIn(advancedTooltipWindow, 250)
  advancedMoveToolTip(true)
end

function g_advancedTooltip.hide()
  g_effects.fadeOut(advancedTooltipWindow, 200)
end

function UIWidget:setAdvancedTooltip(text)
  if text then
  self.advancedTooltip = text
  else
  self.advancedTooltip = true
  end
end

function UIWidget:removeAdvancedTooltip()
  self.advancedTooltip = nil
end

function UIWidget:getAdvancedTooltip()
  return self.advancedTooltip
end

function g_advancedTooltip.setHoveredWidget(widget)
  currentAdvancedHoveredWidget = widget
end

function g_advancedTooltip.getHoveredWidget()
  return currentAdvancedHoveredWidget
end

-- public functions
function g_tooltip.init()
  connect(UIWidget, {  onStyleApply = onWidgetStyleApply,
                       onHoverChange = onWidgetHoverChange})

  addEvent(function()
    toolTipLabel = g_ui.createWidget('UILabel', rootWidget)
    toolTipLabel:setId('toolTip')
    --toolTipLabel:setBackgroundColor('#111111cc')
    toolTipLabel:setImageSource("/images/ui/tooltip_left")
    toolTipLabel:setImageBorderTop(8)
    toolTipLabel:setImageBorderLeft(5)
    toolTipLabel:setImageBorderBottom(5)
    toolTipLabel:setImageBorderRight(6)
    toolTipLabel:setColor("#3E3838")
    toolTipLabel:setTextAlign(AlignCenter)
    toolTipLabel:hide()
    toolTipLabel.onMouseMove = function() moveToolTip() end
  end)
end

function g_tooltip.terminate()
    disconnect(UIWidget, {
        onStyleApply = onWidgetStyleApply,
        onHoverChange = onWidgetHoverChange
    })

    currentHoveredWidget = nil
    toolTipLabel:destroy()
    toolTipLabel = nil

    g_tooltip = nil
end

function g_tooltip.display(text)
    if text == nil or text:len() == 0 then return end
    if not toolTipLabel then return end

    toolTipLabel:setText(text)
    toolTipLabel:resizeToText()
    toolTipLabel:resize(toolTipLabel:getWidth() + 4,
                        toolTipLabel:getHeight() + 4)
    toolTipLabel:show()
    toolTipLabel:raise()
    toolTipLabel:enable()
    g_effects.fadeIn(toolTipLabel, 100)
    moveToolTip(true)

    connect(rootWidget, {onMouseMove = moveToolTip})
end

function g_tooltip.hide()
    g_effects.fadeOut(toolTipLabel, 100)

    disconnect(rootWidget, {onMouseMove = moveToolTip})
end

-- @docclass UIWidget @{

-- UIWidget extensions
function UIWidget:setTooltip(text) self.tooltip = text end

function UIWidget:removeTooltip() self.tooltip = nil end

function UIWidget:getTooltip() return self.tooltip end

-- @}

g_tooltip.init()
g_advancedTooltip.init()
connect(g_app, {onTerminate = g_tooltip.terminate})
